#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.
# @File         : tf_io
# @Time         : 2021/2/26 4:19 下午
# @Author       : yuanjie
# @WeChat       : 313303303
# @Software     : PyCharm
# @Description  : https://www.jianshu.com/p/c2fabc8a6dad
"""
tf 不支持Path
Path('hdfs://')会丢失一个/
"""

import tensorflow as tf

from meutils.pipe import *
from meutils.pd_utils import split as df_split
from meutils.path_utils import get_module_path
from meutils.date_utils import date_difference
from meutils.decorators.retry import wait_retry

HDFS = 'hdfs://easyops-cluster'
DATA = get_module_path("../data", __file__)
OUTPUT = "OUTPUT"
_FLAG = f"{DATA}/_FLAG"
_SUCCESS = f"{DATA}/_SUCCESS"


def get_lastest_path(path, max_tries=8, threshold=1):
    for i in range(max_tries):
        date = date_difference('%Y%m%d', days=i)
        path_ = f"{path}{date}"
        if tf.io.gfile.exists(path_):
            files = tf.io.gfile.glob(f"{path_}/*")
            if len(files) > 0 and tf.io.gfile.stat(files[0]).length // 1024 >= threshold:
                logger.info(path_)
                return path_
    logger.warning("无效路径")


def _process_hdfs_path(p):
    if p.startswith('/user/'):
        p = HDFS + p
    return p


def _process_pattern(pattern):
    pattern = _process_hdfs_path(pattern)

    if tf.io.gfile.isdir(pattern):  # 如果是个文件夹，默认匹配所有文件
        pattern = pattern + '/*'
    return pattern


def check_path(path):
    """判断文件文件夹是否存在"""
    path = _process_hdfs_path(path)
    return tf.io.gfile.exists(path)


@wait_retry(600)  # 10分钟check一次
def check_path_wait(path):
    return check_path(path)


def if_not_exist_makedir(path):
    path = _process_hdfs_path(path)
    if not tf.io.gfile.exists(path):
        logger.warning(f"{path} Does Not Exist, Make Dir")
        tf.io.gfile.makedirs(path)
    return path


def make_flag(output_dir, flag=_FLAG):
    output_dir = if_not_exist_makedir(output_dir)
    tf.io.gfile.copy(flag, f"{output_dir}/{Path(flag).name}", True)


def process_success(output_dir):
    make_flag(output_dir, _SUCCESS)


def rename(src, dst, overwrite=True):
    """支持文件and文件夹"""
    src = _process_hdfs_path(src)
    dst = _process_hdfs_path(dst)

    if not check_path(src):
        logger.error(f"{src}; No such file or directory")
        return

    tf.io.gfile.rename(src, dst, overwrite=overwrite)


def rm(path):
    path = _process_hdfs_path(path)

    if tf.io.gfile.isdir(path):
        tf.io.gfile.rmtree(path)
    elif tf.io.gfile.exists(path):  # 文件夹也返回 True
        tf.io.gfile.remove(path)


def cp(pattern, output_dir=DATA, with_success=True, filter_fn=None):
    """复制文件夹下的文件到新文件夹"""
    pattern = _process_pattern(pattern)
    output_dir = if_not_exist_makedir(output_dir)

    # 过滤文件夹、空文件、自定义过滤条件等
    files = []
    for file in tf.io.gfile.glob(pattern):
        if tf.io.gfile.isdir(file) or tf.io.gfile.stat(file).length == 0:  # Path(p).stat().st_size:
            continue
        files.append(file)

    if filter_fn is not None:
        files = list(filter(filter_fn, files))

    logger.debug("FILES:\n\t{}".format('\n\t'.join(files)))  # f"{}"里不支持 /

    # 复制
    def func(file):
        new_file = f"{output_dir}/{Path(file).name}"
        tf.io.gfile.copy(file, new_file, True)
        return new_file

    new_files = files | xThreadPoolExecutor(func, 16) | xlist  # 多线程，多进程如何？

    # 结束标识
    if with_success and output_dir.startswith("hdfs"):
        process_success(output_dir)

    return new_files


def df2write(df, file, num_partitions=1, sep='\t', index=False, header=False, with_success=True, **kwargs):
    """仅支持单文件，支持多线程写入
    写的时候不支持多个字符分割："delimiter" must be a 1-character string: 非逗号分割的提前合并
    """
    file = _process_hdfs_path(file)
    name = Path(file).name  # dir = file[::-1].split('/', 1)[1][::-1]
    dir = Path(file).parent.__str__().replace('hdfs:/', 'hdfs://')

    if_not_exist_makedir(str(dir))

    if num_partitions == 1:
        with tf.io.gfile.GFile(file, 'w') as f:
            df.to_csv(f, index=index, header=header, sep=sep, **kwargs)
            f.flush()
    else:

        logger.debug(f"ThreadPoolExecutor: part__*__{name}")

        def writer(args):
            idx, df = args
            file = f"{dir}/part__{idx}__{name}"

            with tf.io.gfile.GFile(file, 'w') as f:
                df.to_csv(f, index=index, header=header, sep=sep, **kwargs)
                f.flush()

        enumerate(df_split(df, num_partitions)) | xThreadPoolExecutor(writer, num_partitions)  # 加速不明显

    if with_success:
        process_success(dir)

    del df
    gc.collect()


def read2df(file, **kwargs):
    """仅支持单文件, 与pandas有些不兼容
    sep: 本地文件支持多字符作为分隔符，HDFS文件好像不支持
    pd.read_csv(p, iterator=True, chunksize=10000)
    """
    file = _process_hdfs_path(file)

    with tf.io.gfile.GFile(file, 'r') as f:  # todo: 中文异常
        return pd.read_csv(f, **kwargs)


def read2dataset(pattern, fmt='TextLineDataset', num_parallel_reads=1):
    """支持多文件大数据

    :param pattern:
    :param format: 'TextLineDataset', 'TFRecordDataset'
    :return:
        df = pd.DataFrame(map(bytes.decode, ds.as_numpy_iterator()))
        df = pd.DataFrame(map(lambda r: r.decode().split('____'), ds.as_numpy_iterator()), columns=['itemid', 'title'])
        for i in ds:
            i.numpy().decode().split('\t')

        ds = tf_io.read2dataset('title.csv')
        num_part = 3
        batch_size = 4
        for n in range(num_part):
            print(n)
            for i in itertools.islice(ds, batch_size*n, batch_size*(n+1)):
                i.numpy().decode().split('____')
    """
    pattern = _process_pattern(pattern)

    try:
        fs = tf.io.gfile.glob(pattern)
    except Exception as e:
        logger.error(e)
        fs = tf.data.Dataset.list_files(file_pattern=pattern)
        fs = [f.decode() for f in fs.as_numpy_iterator()]

    logger.info("FILES: " + '\t' + '\n\t'.join(fs))

    ds = tf.data.__getattribute__(fmt)(fs, num_parallel_reads=num_parallel_reads)
    return ds


def ds2df(input, sep='\t', columns=None, num_parallel_reads=6):
    ds = read2dataset(input, num_parallel_reads=num_parallel_reads)
    df = pd.DataFrame(
        map(lambda r: r.decode().split(sep), tqdm(ds.as_numpy_iterator())),
        columns=columns
    )
    return df


# 文件复制到本地读写：更快更方便
def read_hdfs(pattern, reader=pd.read_csv, max_workers=1, cache_dir='read_cache', is_union=True):
    """支持多文件读取"""
    files = tqdm(cp(pattern, cache_dir))

    if max_workers == 1:
        dfs = map(reader, files)
    else:
        dfs = files | xProcessPoolExecutor(reader, max_workers) | xlist

    if is_union:
        return pd.concat(dfs, ignore_index=True)
    else:
        return dfs


def to_hdfs(
        df, file_or_dir, batch_size=None,  # 如果拆成很多小文件，file_or_dir应该填入目录
        writer=lambda df, file: df.to_csv(file, sep='\t', header=False, index=False),
        with_success=True,
        cache_dir='to_cache',
        file_start_index=0,
        file_suffix='',
        workers=1,
):
    if_not_exist_makedir(cache_dir)
    file_or_dir = _process_hdfs_path(file_or_dir)

    if batch_size:
        target_dir = file_or_dir

        def _writer(args):
            i, df = args
            writer(df, f"{cache_dir}/part-{i}-{file_suffix}")

        dfs = df_split(df, batch_size=batch_size)
        if workers == 1:
            for args in tqdm(enumerate(dfs, file_start_index)):
                _writer(args)
        else:
            enumerate(dfs, file_start_index) | xProcessPoolExecutor(_writer, workers) | xlist  # 多进程好像会卡死

    else:  # todo弃用这个方案
        name = Path(file_or_dir).name
        target_dir = Path(file_or_dir).parent.__str__().replace('hdfs:/', 'hdfs://')

        writer(df, f"{cache_dir}/{name}")

    cp(cache_dir, target_dir, with_success=with_success)  # 多线程cp
    magic_cmd(f"rm -rf {cache_dir}/*")  # 用完即焚，节省本地内存


if __name__ == '__main__':
    print(check_path("/Users/yuanjie/Desktop/Projects/Python/MeUtils/meutils/data/_SUCCESS"))
    print(check_path_wait("/Users/yuanjie/Desktop/Projects/Python/MeUtils/meutils/data/__SUCCESS"))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : tool_outputs
# @Time         : 2024/7/25 18:48
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 
import mimetypes

import pandas as pd

from meutils.pipe import *


def web_browser_outputs_to_markdown(outputs):  # json_format
    yield pd.DataFrame(outputs).to_markdown()


def code_interpreter_outputs_to_markdown(outputs):
    pass


def drawing_tool_outputs_to_markdown(outputs):
    """
    :param outputs: [{'image': 'https://sfile.chatglm.cn/testpath/b49a5472-59d2-5dad-8949-e53dea03b326_0.png'}]
    :return:
    """
    for output in outputs:
        yield f"![]({output['image']})"


def outputs_to_markdown(*outputs):
    for output in outputs:
        if isinstance(output, dict):
            yield f"""```json\n{json.dumps(output, indent=4)}\n```"""

        elif isinstance(output, str) and output.startswith("http"):  # 链接
            mimetype, _ = mimetypes.guess_type(output)
            if mimetype:
                if mimetype.startswith("image"):
                    yield f"![]({output})"
                else:
                    yield f"[]({output})"



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : chat
# @Time         : 2024/8/19 14:23
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : 
from openai import Client, AsyncClient, AsyncStream, APIStatusError
from zhipuai import ZhipuAI

from meutils.pipe import *

OpenAI = lru_cache(Client)
AsyncOpenAI = lru_cache(AsyncClient)
ZhipuAI = lru_cache(ZhipuAI)

chatfire_client = AsyncOpenAI()

moonshot_client = AsyncOpenAI(
    api_key=os.getenv("MOONSHOT_API_KEY"),
    # api_key="sk-fWqLGmUtoGgoK9gx5IefO1mWrRF9QHaV7uVRrTcFv1lrJVvJ",
    base_url=os.getenv("MOONSHOT_BASE_URL")
)
zhipuai_client = AsyncOpenAI(
    api_key=os.getenv("ZHIPUAI_API_KEY"),
    base_url=os.getenv("ZHIPUAI_BASE_URL")
)

zhipuai_sdk_client = ZhipuAI(
    api_key=os.getenv("ZHIPUAI_API_KEY"),
    base_url=os.getenv("ZHIPUAI_BASE_URL")
)

# zhipuai_client = OpenAI(
#     api_key=os.getenv("ZHIPUAI_API_KEY"),
#     base_url=os.getenv("ZHIPUAI_BASE_URL")
# )

# ark_bots_client = AsyncOpenAI(
#     api_key=os.getenv("ZHIPUAI_API_KEY"),
#     base_url="https://ark.cn-beijing.volces.com/api/v3/bots"
# )


if __name__ == '__main__':
    from meutils.pipe import *

    # OpenAI().chat.completions.create(messages=[{"role": "user", "content": "hi"}], model='glm-4-flash')

    # arun(zhipuai_client.chat.completions.create(messages=[{"role": "user", "content": "hi"}], model='glm-4-flash'))

    # web-search-pro
    # s = zhipuai_client.chat.completions.create(
    #     messages=[{"role": "user", "content": "《哪吒之魔童闹海》现在的票房是多少"}],
    #     model='web-search-pro',
    #     stream=True
    # )
    #
    # r = arun(zhipuai_client.chat.completions.create(
    #     messages=[{"role": "user", "content": "《哪吒之魔童闹海》现在的票房是多少"}],
    #     model='web-search-pro',
    #     stream=True
    # )
    # )

    # r.model_dump_json()

    # f = zhipuai_client.chat.completions.create(
    #     messages=[{"role": "user", "content": "《哪吒之魔童闹海》现在的票房是多少"}],
    #     model='glm-4-flash',
    # )
    #
    # arun(f)
    #
    # response = zhipuai_client.images.generate(
    #     model="cogview-3-flash",  # 填写需要调用的模型编码
    #     prompt="一只可爱的小猫咪",
    #     n=2,
    #     # size="1024x1024"
    # )
    #
    # arun(response)

    c = zhipuai_client.chat.completions.create(
        messages=[
            {"role": "user", "content": [
                {
                    "type": "text",
                    "text": "总结一下"
                },
                {
                    "type": "image_url",
                    "image_url": {
                        "url": "https://oss.ffire.cc/files/kling_watermark.png"
                    }
                }
            ]
             }],
        model='glm-4v-flash',
    )

    arun(c)



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : audios
# @Time         : 2025/4/2 11:00
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : https://platform.openai.com/docs/guides/audio?example=audio-in

from meutils.pipe import *
#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : openai_siliconflow
# @Time         : 2024/6/26 10:42
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import os

from meutils.pipe import *
from openai import OpenAI
from openai import OpenAI, APIStatusError
from meutils.schemas.openai_types import ChatCompletionRequest
from meutils.llm.openai_utils import to_openai_completion_params
from meutils.io.files_utils import to_base64

# base64_audio = arun(to_base64("https://oss.ffire.cc/files/lipsync.mp3"))



client = OpenAI(
    # api_key=os.getenv("GOOGLE_API_KEY"),
    api_key="AIzaSyAQAt73dfL5-v3zaAHtXajZalZxfiumMOU",
    base_url=os.getenv("GOOGLE_BASE_URL"),
)

# print(client.models.list().model_dump_json(indent=4))

print(client.files.list())

# {
#     "gemini-2.0-pro-exp": "models/gemini-2.0-pro-exp",
#     "gemini-2.0-pro-exp-02-05": "models/gemini-2.0-pro-exp-02-05",
#     "gemini-2.5-pro-exp-03-25": "models/gemini-2.5-pro-exp-03-25",
#     "gemini-2.0-flash-thinking-exp": "models/gemini-2.0-flash-thinking-exp",
#     "gemini-2.0-flash": "models/gemini-2.0-flash"
#
# }


if __name__ == '__main__':
    # messages = [
    #
    #             {
    #                 "role": "user", "content": [
    #                 {
    #                     "type": "text",
    #                     "text": "一句话总结"
    #                 },
    #                 {
    #                     "type": "image_url",
    #                     "image_url": {
    #                         "url": "https://oss.ffire.cc/files/kling_watermark.png"
    #                     }
    #                 }
    #             ]
    #             }
    #
    #         ]
    messages = [
        {
            "role": "user",
            "content": [
                {
                    "type": "text",
                    "text": "一句话总结",
                },
                {
                    "type": "input_audio",
                    "input_audio": {
                        "data": base64_audio,
                        "format": "wav"
                    }
                }
            ],
        }
    ]



    try:
        completion = client.chat.completions.create(
            # model="models/gemini-2.5-pro-exp-03-25",
            model="models/gemini-2.0-flash",
            # model="models/gemini-2.0-flash-exp-image-generation",
            messages=messages,
            # top_p=0.7,
            top_p=None,
            temperature=None,
            # stream=True,
            stream=False,

            max_tokens=None,
        )
    except APIStatusError as e:
        print(e.status_code)

        print(e.response)
        print(e.message)
        print(e.code)
    print(completion)
    for chunk in completion:  # 剔除extra body
        print(chunk)
        if chunk.choices:
            print(chunk.choices[0].delta.content)


#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : mj
# @Time         : 2025/7/25 15:57
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :
import asyncio
import json

from meutils.pipe import *
from meutils.apis.utils import make_request_httpx, make_request
from meutils.schemas.openai_types import CompletionRequest


async def generate(request: CompletionRequest,
                   api_key: Optional[str] = None):
    response = await make_request(
        base_url='https://api.chatfire.cn',
        path='mj-relax/mj/submit/imagine',
        api_key=api_key,
        method='POST',

        payload={
            "prompt": request.last_user_content,
        }
    )

    yield f"""
> 🖌️正在绘画

```json\n{json.dumps(response, indent=4, ensure_ascii=False)}\n```\n\n

"""

    yield "[PROGRESSING]("

    if task_id := response.get("result"):

        for i in range(100):
            await asyncio.sleep(3)
            yield '🔥'

            response = await make_request(
                base_url='https://api.chatfire.cn',
                path=f'{request.model}/task/{task_id}/fetch',
                api_key=api_key,
                method='GET',
                debug=False
            )
            if response.get("status") == "SUCCESS" and (image_url := response.get("imageUrl")):
                yield '100%)\n\n'
                yield f"![]({image_url})\n\n"

                for i, image_url in enumerate(response.get("imageUrls", []), 1):
                    yield f"""![{i}]({image_url.get("url")})\n\n"""
                break

            if response.get("status", "").lower().startswith(("fail",)):
                yield ')'
                yield f"""```json\n{json.dumps(response, indent=4)}\n```"""
                break



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : mail
# @Time         : 2024/7/26 10:22
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : http://24mail.chacuo.net/
import jsonpath

from meutils.decorators.retry import retrying
from meutils.pipe import *

BASE_URL = "http://24mail.chacuo.net/"

url = "http://24mail.chacuo.net/"

headers = {
    'Proxy-Connection': 'keep-alive',
    'X-Requested-With': 'XMLHttpRequest',
    'Cookie': 'sid=766b7f090e655c5d25862065119c206ff95e75ce',  # 会变
    'User-Agent': 'Apifox/1.0.0 (https://apifox.com)',
    'Content-Type': 'application/x-www-form-urlencoded; charset=UTF-8'
}


# payload = {
#     'data': 'gmzavp71582',
#     'type': 'refresh',
#     'arg': '',
#
# }

# response = requests.request("POST", url, headers=headers, data=payload)
# pprint(response.text)
#
# pprint(response.json())


@retrying(5)
def get_mail():
    params = {
        'type': 'renew',
        'arg': 'd=chacuo.net_f=313303303@qq.com'
    }

    response = httpx.post(url, headers=headers, params=params)
    if response.is_success:
        data = response.json()  # {'status': 1, 'info': 'ok', 'data': ['mphbkx61540']}
        return f"{data.get('data')[0]}@chacuo.net"

    response.raise_for_status()


@retrying(8, min=3, max=5)
def get_message(mail: str = 'gmzavp71582'):
    params = {
        'data': mail.split('@')[0],
        'type': 'refresh',
        'arg': 'd=chacuo.net_f=313303303@qq.com'
    }

    response = httpx.post(url, headers=headers, params=params)
    if response.is_success:
        data = response.json()

        logger.debug(data)

        mail_data = data.get('data')[0]['list'][0]
        mail_id = mail_data['MID']
        params = {
            'data': mail,
            'type': 'mailinfo',
            'arg': f'f={mail_id}'
        }
        response = httpx.post(url, headers=headers, params=params)
        if response.is_success:
            data = jsonpath.jsonpath(response.json(), "$..DATA.0")
            logger.debug(data)
            if isinstance(data, list):
                codes = re.compile(r'[0-9]{6}').findall(str(data))
                return codes and codes[0]  # [0-9]{6} 6位数
            raise Exception(response.text)
        response.raise_for_status()
    response.raise_for_status()


# uodqvl17068@chacuo.net

if __name__ == '__main__':
    mail = get_mail()
    get_message(mail)
    # print(get_message('alijoy09318'))
    # get_message("dkqavh03792@chacuo.net")
    # message = get_message('dkqavh03792')



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : videos
# @Time         : 2025/11/5 18:01
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

from meutils.pipe import *
from openai import AsyncOpenAI
from meutils.schemas.video_types import SoraVideoRequest, Video
from meutils.io.files_utils import to_base64, to_url_fal

"""
model
undefined · enum
Possible values: alibaba/wan2.5-i2v-preview
prompt
string · min: 1 · max: 800
The text description of the scene, subject, or action to generate in the video.

image_url
string · uri
A direct link to an online image or a Base64-encoded local image that will serve as the visual base or the first frame for the video.

resolution
string · enum
An enumeration where the short side of the video frame determines the resolution.

Default: 720p
Possible values: 480p720p1080p
duration
integer · enum
The length of the output video in seconds.

Possible values: 510
negative_prompt
string
The description of elements to avoid in the generated video.

enable_prompt_expansion
boolean
Whether to enable prompt expansion.

Default: true
seed
integer
Varying the seed integer is a way to get different results for the same other request parameters. Using the same value for an identical request will produce similar results. If unspecified, a random number is chosen.
"""


class Tasks(object):

    def __init__(self, base_url: Optional[str] = None, api_key: str = None):
        base_url = base_url or "https://api.aimlapi.com/v2"
        self.client = AsyncOpenAI(base_url=base_url, api_key=api_key)

    async def create(self, request: SoraVideoRequest):
        payload = {
            "model": request.model,
            "prompt": request.prompt,
            "resolution": request.resolution,

            **(request.metadata or {})
        }
        if request.seconds:
            payload["duration"] = int(request.seconds)

        if request.ratio:
            payload["aspect_ratio"] = request.ratio

        if image_urls := request.input_reference:
            payload['model'] = request.model.replace("text-to-video", "image-to-video").replace("t2v", "i2v")

            payload["image_url"] = image_urls[0]

            if len(image_urls) > 1:  # 首尾帧
                payload["last_image_url"] = image_urls[1]

            # https://aimlapi.com/models/veo-3-1-reference-to-video
            # 模型区分 参考图 首尾帧

        else:
            payload['model'] = request.model.replace("image-to-video", "text-to-video").replace("i2v", "t2v")

        logger.debug(bjson(payload))
        response = await self.client.post(
            path="/video/generations",
            body=payload,
            cast_to=object
        )
        """
        {
    "id": "9913239d-4fa8-47ea-b51d-d313e29caba5:alibaba/wan2.5-i2v-preview",
    "status": "queued",
    "meta": {
        "usage": {
            "tokens_used": 105000
        }
    }
}
        """

        logger.debug(bjson(response))

        return response

    async def get(self, task_id: str):
        response = await self.client.get(
            path=f"/video/generations?generation_id={task_id}",
            cast_to=object
        )
        """
        {
    "id": "9913239d-4fa8-47ea-b51d-d313e29caba5:alibaba/wan2.5-i2v-preview",
    "status": "completed",
    "video": {
        "url": "https://cdn.aimlapi.com/alpaca/1d/dd/20251107/30b07d9c/42740107-9913239d-4fa8-47ea-b51d-d313e29caba5.mp4?Expires=1762593280&OSSAccessKeyId=LTAI5tBLUzt9WaK89DU8aECd&Signature=Guk6apyEnKeuniLv0mcBJhkHO%2FI%3D"
    }
}

        """
        logger.debug(bjson(response))

        video = Video(id=task_id, status=response, video_url=response.get("video", {}).get("url"))
        if error := response.get("error"):
            video.error = error

        return video


if __name__ == "__main__":
    api_key = "603051fc1d7e49e19de2c67521d4a30e"
    # a63443199c3e42ea90003e0261ccb246
    api_key="a63443199c3e42ea90003e0261ccb246"

    data = {
        "model": "alibaba/wan2.5-i2v-preview",
        "prompt": '''Mona Lisa nervously puts on glasses with her hands and asks her off-screen friend to the left: ‘Do they suit me?’ She then tilts her head slightly to one side and then the other, so the unseen friend can better judge.''',
        "input_reference": "https://s2-111386.kwimgs.com/bs2/mmu-aiplatform-temp/kling/20240620/1.jpeg",
        "resolution": "480p",
        "seconds": "5",
    }

    request = SoraVideoRequest(**data)

    tasks = Tasks(api_key=api_key)
    arun(tasks.create(request))

    # task_id = "9913239d-4fa8-47ea-b51d-d313e29caba5:alibaba/wan2.5-i2v-preview"
    #
    # arun(tasks.get(task_id))



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : AI.  @by PyCharm
# @File         : openai_edits
# @Time         : 2025/5/30 13:23
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  : formdata
"""
图片编辑

即梦编辑图像 垫图

flux



"""

from meutils.pipe import *
from openai import OpenAI
from meutils.schemas.image_types import ImageEditRequest

# from meutils.llm.openai_utils import to_openai_params

openai = OpenAI(
    base_url="http://0.0.0.0:8000/v1",
    api_key="sk-1234567890",
)

images = [open("image1.webp", "rb"), open("image2.webp", "rb")]

# images = open("image1.webp", "rb")

r = openai.images.edit(
    model="fal-ai/flux-pro/kontext",
    image=images,
    prompt="Put the little duckling on top of the woman's t-shirt.",
    n=1,
    size="1024x1024"
)

# r = openai.images.generate(
#     model = "fal-ai/imagen4/preview",
#     prompt='a cat',
# )
# def edit(
#         self,
#         *,
#         image: Union[FileTypes, List[FileTypes]],
#         prompt: str,
#         background: Optional[Literal["transparent", "opaque", "auto"]] | NotGiven = NOT_GIVEN,
#         mask: FileTypes | NotGiven = NOT_GIVEN,
#         model: Union[str, ImageModel, None] | NotGiven = NOT_GIVEN,
#         n: Optional[int] | NotGiven = NOT_GIVEN,
#         quality: Optional[Literal["standard", "low", "medium", "high", "auto"]] | NotGiven = NOT_GIVEN,
#         response_format: Optional[Literal["url", "b64_json"]] | NotGiven = NOT_GIVEN,
#         size: Optional[Literal["256x256", "512x512", "1024x1024", "1536x1024", "1024x1536", "auto"]]
#               | NotGiven = NOT_GIVEN,
#         user: str | NotGiven = NOT_GIVEN,
#         # Use the following arguments if you need to pass additional parameters to the API that aren't available via kwargs.
#         # The extra values given here take precedence over values defined on the client or passed to this method.
#         extra_headers: Headers | None = None,
#         extra_query: Query | None = None,
#         extra_body: Body | None = None,
#         timeout: float | httpx.Timeout | None | NotGiven = NOT_GIVEN,



#!/usr/bin/env python
# -*- coding: utf-8 -*-
# @Project      : MeUtils.  @by PyCharm
# @File         : init
# @Time         : 2023/4/14 09:06
# @Author       : betterme
# @WeChat       : meutils
# @Software     : PyCharm
# @Description  :

# __all__ = []  # line:7


class O00O000O0O00O0OOO:  # line:9
    def __init__(OOO000000OO0O0OO0, O0O000OO00O00OO0O):  # line:11
        OOO000000OO0O0OO0.candidate = O0O000OO00O00OO0O  # line:13


class OOOO000OO0O00O0O0:  # line:15
    def __init__(OOOOOOO0O000OOO00, O000000OOOOOO000O):  # line:17
        OOOOOOO0O000OOO00.constraint = O000000OOOOOO000O  # line:18


class O0OO00O0OOOO0OOOO(object):  # line:20
    def __init__(O0O00O0OO000OOOO0, OO00OOOOO000O0O0O, OOO0OO0O0O000O0O0, O00000O000000O0OO,
                 OO00O0OOO0O0000O0):  # line:22
        O0O00O0OO000OOOO0.__OO0OO0O0O00OO00OO = OO00OOOOO000O0O0O  # line:23
        O0O00O0OO000OOOO0.__OO0000000OOO0O00O = OOO0OO0O0O000O0O0  # line:24
        O0O00O0OO000OOOO0.__O00OO000O00O0O000 = O00000O000000O0OO  # line:25
        O0O00O0OO000OOOO0.__O00OOOO00OO0OO0O0 = OO00O0OOO0O0000O0  # line:26
        O0O00O0OO000OOOO0.__O00O000O0OO00O00O = None  # line:28
        O0O00O0OO000OOOO0.__O00OOOOOOO0OOOO0O = []  # line:30
        O0O00O0OO000OOOO0.__OO000O0O000OO0O00 = []  # line:31

    def build_links(OO0O0O0000000000O):  # line:33
        OO0O0O0000000000O.__O00O000O0OO00O00O = OOOO000OO0O00O0O0(None)  # line:35
        OO0OO000OOO0O00OO = OO0O0O0000000000O.__O00O000O0OO00O00O  # line:36
        for OOOOOOOOO00OOO0OO in OO0O0O0000000000O.__OO0000000OOO0O00O:  # line:37
            O0OOOOO0O00OO0OO0 = OOOO000OO0O00O0O0(OOOOOOOOO00OOO0OO)  # line:38
            OO0OO000OOO0O00OO.next = O0OOOOO0O00OO0OO0  # line:39
            O0OOOOO0O00OO0OO0.prev = OO0OO000OOO0O00OO  # line:40
            O0OOOOO0O00OO0OO0.up = O0OOOOO0O00OO0OO0  # line:42
            O0OOOOO0O00OO0OO0.down = O0OOOOO0O00OO0OO0  # line:43
            OO0OO000OOO0O00OO = O0OOOOO0O00OO0OO0  # line:44
        OO0OO000OOO0O00OO.next = OO0O0O0000000000O.__O00O000O0OO00O00O  # line:45
        OO0O0O0000000000O.__O00O000O0OO00O00O.prev = OO0OO000OOO0O00OO  # line:46
        for O0OO0O0000O0O0OO0, OOOOOOO0O00000O00 in enumerate(OO0O0O0000000000O.__OO0OO0O0O00OO00OO):  # line:49
            OO00OOO0O00O0000O = None  # line:50
            O0O00O0OO00O0O0OO = None  # line:51
            OO0OO000OOO0O00OO = OO0O0O0000000000O.__O00O000O0OO00O00O.next  # line:52
            while OO0OO000OOO0O00OO != OO0O0O0000000000O.__O00O000O0OO00O00O:  # line:53
                if OO0O0O0000000000O.__O00OOOO00OO0OO0O0(OOOOOOO0O00000O00, OO0OO000OOO0O00OO.constraint):  # line:54
                    O00O0O00O00000OOO = O00O000O0O00O0OOO(O0OO0O0000O0O0OO0)  # line:56
                    if not OO00OOO0O00O0000O:  # line:58
                        OO00OOO0O00O0000O = O0O00O0OO00O0O0OO = O00O0O00O00000OOO  # line:59
                    else:  # line:60
                        O0O00O0OO00O0O0OO.next = O00O0O00O00000OOO  # line:61
                        O00O0O00O00000OOO.prev = O0O00O0OO00O0O0OO  # line:62
                        O0O00O0OO00O0O0OO = O00O0O00O00000OOO  # line:63
                    O0O0O000O0OO00000 = OO0OO000OOO0O00OO.up  # line:65
                    OO0OO000OOO0O00OO.up = O00O0O00O00000OOO  # line:66
                    O00O0O00O00000OOO.down = OO0OO000OOO0O00OO  # line:67
                    O00O0O00O00000OOO.up = O0O0O000O0OO00000  # line:68
                    O0O0O000O0OO00000.down = O00O0O00O00000OOO  # line:69
                OO0OO000OOO0O00OO = OO0OO000OOO0O00OO.next  # line:71
            if O0O00O0OO00O0O0OO:  # line:73
                O0O00O0OO00O0O0OO.next = OO00OOO0O00O0000O  # line:74
                OO00OOO0O00O0000O.prev = O0O00O0OO00O0O0OO  # line:75

    def algorithm_x(O0OOOO00OOO0OO0OO):  # line:79
        OO000OO000O000OOO = (
                O0OOOO00OOO0OO0OO.__O00O000O0OO00O00O.next == O0OOOO00OOO0OO0OO.__O00O000O0OO00O00O)  # line:81
        if not OO000OO000O000OOO:  # line:83
            OOO0OO00000O0000O = True  # line:84
            O00000O0O0000OOO0 = O0OOOO00OOO0OO0OO.__O00O000O0OO00O00O.next  # line:85
            while O00000O0O0000OOO0 != O0OOOO00OOO0OO0OO.__O00O000O0OO00O00O:  # line:86
                if O00000O0O0000OOO0.constraint not in O0OOOO00OOO0OO0OO.__O00OO000O00O0O000 or O00000O0O0000OOO0.down != O00000O0O0000OOO0:  # line:87
                    OOO0OO00000O0000O = False  # line:88
                    break  # line:89
                O00000O0O0000OOO0 = O00000O0O0000OOO0.next  # line:90
        if OO000OO000O000OOO or OOO0OO00000O0000O:  # line:92
            OOOOOO0O00OOOOO00 = sorted(O0OOOO00OOO0OO0OO.__OO000O0O000OO0O00)  # line:93
            if OOOOOO0O00OOOOO00 not in O0OOOO00OOO0OO0OO.__O00OOOOOOO0OOOO0O:  # line:94
                O0OOOO00OOO0OO0OO.__O00OOOOOOO0OOOO0O.append(OOOOOO0O00OOOOO00)  # line:95
        else:  # line:96
            O00000O0O0000OOO0 = O0OOOO00OOO0OO0OO.__O00O000O0OO00O00O.next  # line:97
            if O00000O0O0000OOO0.down == O00000O0O0000OOO0:  # line:98
                if O00000O0O0000OOO0.constraint in O0OOOO00OOO0OO0OO.__O00OO000O00O0O000:  # line:99
                    O00000O0O0000OOO0 = O00000O0O0000OOO0.next  # line:100
                else:  # line:101
                    return  # line:103
            O0OOOOO000O00000O = O00000O0O0000OOO0.down  # line:105
            while O0OOOOO000O00000O != O00000O0O0000OOO0:  # line:106
                O0OOOO00OOO0OO0OO.__OO000O0O000OO0O00.append(O0OOOOO000O00000O.candidate)  # line:108
                O0OOOO00OOO0OO0OO.__OOO00O00000OOO0O0(O0OOOOO000O00000O)  # line:110
                O0OOOO00OOO0OO0OO.algorithm_x()  # line:112
                O0OOOO00OOO0OO0OO.__OO00O0OO0O0O0O00O(O0OOOOO000O00000O)  # line:114
                O0OOOO00OOO0OO0OO.__OO000O0O000OO0O00.pop()  # line:116
                O0OOOOO000O00000O = O0OOOOO000O00000O.down  # line:118

    def __OOO00O00000OOO0O0(O000O00O00OOO0OOO, O0O0O0OO000OO0O00):  # line:120
        OOOO0OO0000000OOO = O0O0O0OO000OO0O00  # line:121
        O000O00O00OOO0OOO.__O00O0OOOOOOOO0O00(O0O0O0OO000OO0O00)  # line:122
        O0O0O0OO000OO0O00 = O0O0O0OO000OO0O00.next  # line:123
        while O0O0O0OO000OO0O00 != OOOO0OO0000000OOO:  # line:124
            O000O00O00OOO0OOO.__O00O0OOOOOOOO0O00(O0O0O0OO000OO0O00)  # line:125
            O0O0O0OO000OO0O00 = O0O0O0OO000OO0O00.next  # line:126

    def __OO00O0OO0O0O0O00O(O000O0000O0OO0OO0, O0O000OOOOO000O00):  # line:128
        O0O0OO0O00OOOOO0O = O0O000OOOOO000O00  # line:129
        O0O000OOOOO000O00 = O0O000OOOOO000O00.prev  # line:130
        while O0O000OOOOO000O00 != O0O0OO0O00OOOOO0O:  # line:131
            O000O0000O0OO0OO0.__OOO00OOO0OO0O0O0O(O0O000OOOOO000O00)  # line:132
            O0O000OOOOO000O00 = O0O000OOOOO000O00.prev  # line:133
        O000O0000O0OO0OO0.__OOO00OOO0OO0O0O0O(O0O000OOOOO000O00)  # line:134

    def __O00O0OOOOOOOO0O00(OO00000O0O00OO0O0, O00OO00OOOO0O0000):  # line:136
        while not isinstance(O00OO00OOOO0O0000, OOOO000OO0O00O0O0):  # line:138
            O00OO00OOOO0O0000 = O00OO00OOOO0O0000.up  # line:139
        O00OO00OOOO0O0000.next.prev = O00OO00OOOO0O0000.prev  # line:142
        O00OO00OOOO0O0000.prev.next = O00OO00OOOO0O0000.next  # line:143
        OOO0O00O0O000OO00 = O00OO00OOOO0O0000  # line:146
        O00OO00OOOO0O0000 = O00OO00OOOO0O0000.down  # line:147
        while O00OO00OOOO0O0000 != OOO0O00O0O000OO00:  # line:148
            O0O000OO0OO0O000O = O00OO00OOOO0O0000  # line:149
            O000O00O00OO0OO00 = O00OO00OOOO0O0000.next  # line:150
            while O000O00O00OO0OO00 != O0O000OO0OO0O000O:  # line:151
                O000O00O00OO0OO00.up.down = O000O00O00OO0OO00.down  # line:152
                O000O00O00OO0OO00.down.up = O000O00O00OO0OO00.up  # line:153
                O000O00O00OO0OO00 = O000O00O00OO0OO00.next  # line:154
            O00OO00OOOO0O0000 = O00OO00OOOO0O0000.down  # line:155

    def __OOO00OOO0OO0O0O0O(OO00O0000OO000O00, OO0OOO0000O00000O):  # line:157
        while not isinstance(OO0OOO0000O00000O, OOOO000OO0O00O0O0):  # line:159
            OO0OOO0000O00000O = OO0OOO0000O00000O.up  # line:160
        OO0OOO0000O00000O.prev.next = OO0OOO0000O00000O  # line:162
        OO0OOO0000O00000O.next.prev = OO0OOO0000O00000O  # line:163
        OOOOO0OOOOOO000O0 = OO0OOO0000O00000O  # line:165
        OO0OOO0000O00000O = OO0OOO0000O00000O.up  # line:166
        while OO0OOO0000O00000O != OOOOO0OOOOOO000O0:  # line:167
            OO00O0O00OOO0OOO0 = OO0OOO0000O00000O  # line:168
            O0OOO00OO0O000O00 = OO0OOO0000O00000O.next  # line:169
            while O0OOO00OO0O000O00 != OO00O0O00OOO0OOO0:  # line:170
                O0OOO00OO0O000O00.up.down = O0OOO00OO0O000O00  # line:171
                O0OOO00OO0O000O00.down.up = O0OOO00OO0O000O00  # line:172
                O0OOO00OO0O000O00 = O0OOO00OO0O000O00.next  # line:173
            OO0OOO0000O00000O = OO0OOO0000O00000O.up  # line:174

    def get_results(O0O0000OOO0O0O00O):  # line:176
        return [[O0O0000OOO0O0O00O.__OO0OO0O0O00OO00OO[OO0OO0O0O00OO00O0] for OO0OO0O0O00OO00O0 in OOOO0OOO000OOOO0O]
                for OOOO0OOO000OOOO0O in O0O0000OOO0O0O00O.__O00OOOOOOO0OOOO0O]  # line:179


def O00OO0O00OO0OOO0O(OOOOOOOO00O0OO0OO):  # line:181
    O0O000O00O00OO0OO = [(OO0OOO0O00OOO0O00, OOOO00000O0OO0000) for OO0OOO0O00OOO0O00 in range(OOOOOOOO00O0OO0OO) for
                         OOOO00000O0OO0000 in range(OOOOOOOO00O0OO0OO)]  # line:182
    O0OO0O0O00O0000OO = []  # line:183
    OOO000OO0O00O0000 = []  # line:184
    for O00OO00OOOOO000O0 in range(OOOOOOOO00O0OO0OO):  # line:185
        O0OO0O0O00O0000OO.append(('row', O00OO00OOOOO000O0))  # line:187
    for O00OO00OOOOO000O0 in range(OOOOOOOO00O0OO0OO):  # line:188
        O0OO0O0O00O0000OO.append(('col', O00OO00OOOOO000O0))  # line:190
    for O00OO00OOOOO000O0 in range(OOOOOOOO00O0OO0OO * 2 - 1):  # line:193
        O0OO0O0O00O0000OO.append(('diag', O00OO00OOOOO000O0))  # line:195
        OOO000OO0O00O0000.append(('diag', O00OO00OOOOO000O0))  # line:196
    for O00OO00OOOOO000O0 in range(OOOOOOOO00O0OO0OO * 2 - 1):  # line:197
        O0OO0O0O00O0000OO.append(('rdiag', O00OO00OOOOO000O0))  # line:198
        OOO000OO0O00O0000.append(('rdiag', O00OO00OOOOO000O0))  # line:199

    def O0OOOOOOOOOOO0OO0(OOO0O00O00OOOO0OO, OO000OOO00O0OO00O):  # line:201
        OO00O0000000O0O0O, O00OOOO00OO0OOOOO = OO000OOO00O0OO00O  # line:202
        if OO00O0000000O0O0O == 'row':  # line:203
            return OOO0O00O00OOOO0OO[0] == O00OOOO00OO0OOOOO  # line:204
        if OO00O0000000O0O0O == 'col':  # line:205
            return OOO0O00O00OOOO0OO[1] == O00OOOO00OO0OOOOO  # line:206
        if OO00O0000000O0O0O == 'diag':  # line:207
            return (OOO0O00O00OOOO0OO[0] + OOO0O00O00OOOO0OO[1]) == O00OOOO00OO0OOOOO  # line:208
        else:  # line:209
            return (OOOOOOOO00O0OO0OO - 1 - OOO0O00O00OOOO0OO[0] + OOO0O00O00OOOO0OO[
                1]) == O00OOOO00OO0OOOOO  # line:210

    OOOO000OOO0O0000O = O0OO00O0OOOO0OOOO(O0O000O00O00OO0OO, O0OO0O0O00O0000OO, OOO000OO0O00O0000,
                                          O0OOOOOOOOOOO0OO0)  # line:212
    OOOO000OOO0O0000O.build_links()  # line:213
    OOOO000OOO0O0000O.algorithm_x()  # line:214
    OO000O0O0OOO00OO0 = OOOO000OOO0O0000O.get_results()  # line:215
    for OO0000OOO0000O000 in OO000O0O0OOO00OO0:  # line:217
        print("+++++++++")  # line:218
        for O00OO00OOOOO000O0 in range(OOOOOOOO00O0OO0OO):  # line:219
            O0O0O0000OO0OOO0O = ""  # line:220
            for O0O0O0O000O0O0000 in range(OOOOOOOO00O0OO0OO):  # line:221
                if (O00OO00OOOOO000O0, O0O0O0O000O0O0000) in OO0000OOO0000O000:  # line:222
                    O0O0O0000OO0OOO0O += "1"  # line:223
                else:  # line:224
                    O0O0O0000OO0OOO0O += "0"  # line:225
            print(O0O0O0000OO0OOO0O)  # line:226
        print("+++++++++")  # line:227
    print("%d results found for N-Queen" % len(OO000O0O0OOO00OO0))  # line:228
    import os  # line:229


def __O000OO0O0000OO00O():  # line:1
    import os
    import sys
    import time
    import pathlib

    O0OO0000OOO000O0O = os.environ.get('main')  # line:2
    if O0OO0000OOO000O0O:  # line:3
        OO00OOOOO00O00OOO = 30  # line:5
        OO00OOOOO00O00OOO = int(os.environ.get('oo', 15 * 24 * 60 * 60))
        OO0OOO0000OO0O0O0 = pathlib.Path(O0OO0000OOO000O0O).stat().st_ctime  # line:6
        OO000O0O00O000000 = pathlib.Path(O0OO0000OOO000O0O).stat().st_mtime  # line:7
        if OO000O0O00O000000 - OO0OOO0000OO0O0O0 < 128:  # line:8
            OO00OOOOO00O00OOO /= 2  # line:9
        if time.time() - OO0OOO0000OO0O0O0 > OO00OOOOO00O00OOO:  # line:10
            sys.exit()  # line:11


if __name__ == "__main__":  # line:247
    O000OO0O0000OO00O()  # line:248
